"""
This module contains functionality for managing backups on the
PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> b = pl.Backup()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import tempfile

from resource import Resource
from thisversion import PLDB

import plutil

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

class Backup(Resource):
	"""
	A connection to a PacketLogic's "Backup" resource.

	The methods of this object allows listing, taking and
	restoring backups on the PacketLogic system.

	@group Backup: list,take,restore
	"""
	def _resource(self):
		return PLDB.RESOURCE_BACKUPS

	def list(self):
		"""
		List backups stored on server.

			>>> b.list()
			[{'cdate': '', 'path': '20090520-05.14.plb', 'size': 353714L},
			 {'cdate': '', 'path': '20090731-22.52.plb', 'size': 272156L},
			 {'cdate': '', 'path': '20090831-09.56.plb', 'size': 177219L},
			 {'cdate': '', 'path': '20090916-16.24.plb', 'size': 596406L}]

		@rtype: C{list} of C{dict}
		@returns: a list of backups

		"""

		return plutil._do(self, self.session.backup_list(), plutil._ListCB(self, self.session.backup_list_handle, None, False))

	def take(self, name="", resource=""):
		"""
		Take a new backup.

		Creates a new backup of current database contents. The
		backup will be given the current date and time as
		name.

			>>> b.take()
			'20091026-08.15.plb'

		@type name: C{str}
		@param name: Name of newly created backup file. Requires firmware v12.4 or newer.

		@type resource: C{str}
		@param resource: Include only the specified resource
		                 in backup file, use empty string ""
		                 to include all resources
		                 (default). Requires firmware v12.4 or
		                 newer.

		@rtype: C{str}
		@returns: Name of new backup

		@raises PLDBUnsupportedInFirmware: If 'name' or 'resource' argument is used without firmware support.
		"""
		return plutil._do(self, self.session.backup_dump(name, resource), plutil._AddCB(self, self.session.backup_dump_handle))

	def restore(self, name, section="All", systemid="local"):
		"""
		Restore a backup

			>>> b.restore("mybackup.plb", "Rules & Object Configuration")

		@type name: C{str}
		@param name: Name of backup file

		@type section: C{str}
		@param section: The section in the backup file to restore,
		                such as "Rules & Object Configuration" or
		                "All" for all sections (default).

		@type systemid: C{str}
		@param systemid: Systemid to use for the items, or
		                 "local" to use system's default, or
		                 "file" to use the systemid in file.

		@raises PLDBUnsupportedInFirmware: If 'systemid' is used in firmware without support.
		"""
		if systemid != 'local':
			raise PLDBUnsupportedInFirmware("Usage of 'machineid' other than 'local' is not supported in this firmware version")
		return plutil._do(self, self.session.backup_restore(name, section), plutil._AddCB(self))

	def remove(self, name):
		"""
		Remove a backup file

			>>> b.restore("mybackup.plb")

		@type name: C{str}
		@param name: Name of backup file

		"""
		return plutil._do(self, self.session.backup_remove(name), plutil._AddCB(self))

	def download(self, name):
		"""
		Download specified backup file.

			>>> plb = b.download("20090916-16.24.plb")
			>>> len(plb)
			596406L
			>>> plb[:52]
			"<?xml version='1.0' encoding='UTF-8'?>\\n<PacketLogic "

		@type name: C{str}
		@param name: Name of backup to download.

		@rtype: C{str}
		@returns: The contents of the requested backup.
		"""
		ofile = tempfile.TemporaryFile()
		size = self._backup_download(name, ofile)
		ofile.flush()
		ofile.seek(0)
		data = ofile.read()
		ofile.close()
		assert len(data) == size

		return data

	def download_to(self, name, destpath):
		"""
		Download specified backup file to a local file

			>>> download_to("20090916-16.24.plb", "/tmp/back.plb")

		@type name: C{str}
		@param name: Name of backup to download.

		@type destpath: C{str}
		@param destpath: Destination path for downloaded file
		"""
		ofile = file(destpath, "wb")
		size = self._backup_download(name, ofile)
		ofile.flush()
		ofile.close()

	def upload(self, fname, data):
		"""
		Upload a backup

			>>> data = file("/tmp/back.plb","rb").read()
			>>> len()
			596406L
			>>> data[:52]
			"<?xml version='1.0' encoding='UTF-8'?>\\n<PacketLogic "
			>>> b.upload("name-on-pl.plb", data)

		@type fname: C{str}
		@param fname: Name of file.

		@type data: C{str}
		@param data: The contents of the file.
		"""

		ifile = tempfile.TemporaryFile()
		ifile.write(data)
		ifile.flush()
		ifile.seek(0)
		self._backup_upload(fname, ifile)

	def upload_from(self, fname, srcpath):
		"""
		Upload a backup from a local file.

			>>> b.upload_from("name-on-pl.plb", "/tmp/back.plb")

		@type fname: C{str}
		@param fname: Name of file.

		@type srcpath: C{str}
		@param srcpath: Path to source file
		"""

		ifile = file(srcpath, "rb")
		self._backup_upload(fname, ifile)
		ifile.close()

	def _backup_download (self, name, ofile):
		assert hasattr(ofile, 'write')

		def callback (msg):
			if msg.get_type() == PLDB.MSG_TYPE_DATA:
				ok, callback.size = self.session.backup_download_handle(
					msg, ofile, callback.size)
				if not ok:
					raise RuntimeError("Download handler failed")
			callback.done = msg.get_type() == PLDB.MSG_TYPE_OK
			return callback.done

		callback.size = 0
		callback.done = False

		cid = self.session.backup_download(name)
		self._addcidcb(cid, callback)

		while not callback.done:
			self._process()

		return callback.size


	def _backup_upload (self, fname, ifile):
		assert hasattr(ifile, 'read')
		cid, fil = self.session.backup_upload(fname, ifile)

		def callback (msg):
			cid = self.session.backup_upload_handle(msg, fil)
			if cid:
				self._addcidcb(cid, callback)
			else:
				callback.done = True
		callback.done = False

		self._addcidcb(cid, callback)
		while not callback.done:
			self._process()


