"""
This module contains functionality for managing channels on the
PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> ch = pl.Channels()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

from resource import Resource
from thisversion import PLDB

import plutil
from packetlogic2.exceptions import PLDBUnsupportedInFirmware

_Prop_Channel = plutil.Prop('id', 'label', 'location', 'internal', 'internal_media', 'external',
			    'external_media', 'type', 'caps', 'role', 'direction', active=bool)

class Channel(object):
	""" Information about a Channel.

	Returned in a list by L{Channels.channel_list}.

	@type id: C{int}
	@type active: C{bool}
	@type label: C{str}
	@type location: C{str}
	@type description: C{str}
	@type internal: C{str}
	@type internal_media: C{int}
	@type external: C{str}
	@type external_media: C{int}

	@cvar __slots__: For internal use only.
	"""
	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		"""
		Should not be instantiated manually, use L{channel_list <Channels.channel_list>}.
		"""
		self._pldb = _pldb
		self._prop = _Prop_Channel(**props)

	def __repr__(self):
		return "<%s #%d '%s'>" % (self.__class__.__name__, self._prop.id, self._prop.location)

	def set_active(self, active):
		""" Set or reset the active state on this Channel.

		@type active: C{bool}
		@param active: The active state to set on this Channel.

		"""
		active = bool(active)
		plutil._do(self._pldb, self._pldb.session.channel_set_active(self._prop.id, active), plutil._AddCB(self._pldb))
		self._prop.active = active

	def update(self, internal_media=None, external_media=None, role=None, label=None, direction=None):
		""" Update properties on this Channel.
		Omitting an argument or setting it to C{None} will leave that property unchanged.

		@type internal_media: C{int}
		@param internal_media: One of the CHANNEL_MEDIA_ constants.

		@type external_media: C{int}
		@param external_media: One of the CHANNEL_MEDIA_ constants.

		@type role: C{int}
		@param role: One of the ROLE_ constants.

		@type label: C{str}
		@param label: Label for this channel.

		@type direction: C{int}
		@param direction: One of the DIRECTION_ constants.

		"""
		if internal_media is None:
			internal_media = self._prop.internal_media

		if external_media is None:
			external_media = self._prop.external_media

		if role is None:
			role = self._prop.role
		if label is None:
			label = self._prop.label
		if direction is None:
			direction = self._prop.direction

		plutil._do(self._pldb, self._pldb.session.channel_update(self, label,
									 internal_media, external_media,
									 role, direction), plutil._AddCB(self._pldb))

		self._prop.internal_media = internal_media
		self._prop.external_media = external_media
		self._prop.role = role
		self._prop.label = label
		self._prop.direction = direction

	id = property(lambda self: self._prop.id, doc="ID Number of the channel.")
	label = property(lambda self: self._prop.label,
                         doc="""The channel's label.

                             Label used for referencing this channel
                             from filtering rules.

                             @attention: label requires firmware 14.1 or newer.

                             """)
	location = property(lambda self: self._prop.location,
                            doc="""The channel's location.

                                A string describing the channels location.
                                e.g: "Channel 1/2"
                                """)
	description = property(lambda self: self._prop.location,
                               doc="Deprecated alias for 'location'.")

	active = property(lambda self: self._prop.active, doc="Active state.")
	internal = property(lambda self: self._prop.internal, doc="System name of the internal interface.")
	internal_media = property(lambda self: self._prop.internal_media, doc="Media type on Internal interface. One of the MEDIA_ constants.")
	external = property(lambda self: self._prop.external, doc="System name of the external interface.")
	external_media = property(lambda self: self._prop.external_media, doc="Media type on External interface. One of the MEDIA_ constants.")
	type = property(lambda self: self._prop.type, doc="Channel type. One of the TYPE_ constants.")
	caps = property(lambda self: self._prop.caps, doc="Capabilities bitmask. One or more of the CAPS_ constants.")
	role = property(lambda self: self._prop.role, doc="Channel role. One of the ROLE_ constants.")
	direction = property(lambda self: self._prop.direction, doc="Channel direaction. One of the DIRECTION_ constants.")

_Prop_ChannelLabel = plutil.Prop('id', 'flags', 'name', 'channels')

class ChannelLabel(object):
	""" Information about a Channel label.

	Returned in a list by L{Channels.channel_divert_label_list} or L{Channels.channel_monitor_label_list}.

	"""

	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		"""
		Should not be instantiated manually, use L{Channels.channel_divert_label_list} or L{Channels.channel_monitor_label_list}.
		"""
		self._pldb = _pldb
		self._prop = _Prop_ChannelLabel(**props)
		for clc in self._prop.channels:
			clc._channellabel = self

	def __repr__(self):
		return "<%s %s>" % (self.__class__.__name__, self._prop.name)

	def _update_channels(self):
		# This function is called from ChannelLabelChannels.update whenever a property is updated in the channels list
		self._pldb.session.channel_label_channels_set (self._prop.id, self._prop.channels)

	def update(self, name=None, flags=None):
		""" Update properties on this ChannelLabel.
		Omitting an argument or setting it to C{None} will leave that property unchanged.

		"""
		if name is not None:
			self._prop.name = name
		if flags is not None:
			self._prop.flags = flags

		self._pldb.session.channel_label_rename (self._prop.id, self._prop.name, self._prop.flags)

	def remove(self):
		""" Remove this ChannelLabel
		"""
		self._pldb.session.channel_label_remove (self._prop.id)

	def add_channel(self, channelid):
		""" Add ChannelLabel to specified channel id.
		The role of the specified channel must be the same as the role specified
		for the ChannelLabel.
		"""
		temp_clc = ChannelLabelChannel()
		temp_clc._prop.vlan = 0
		temp_clc._prop.flags = 0
		temp_clc._prop.channel = channelid
		# These defaults are from the sql file, hopefully they will not change over time
		temp_clc._prop.int_address = "169.254.250.1/24"
		temp_clc._prop.int_gateway = "169.254.250.254"
		temp_clc._prop.int_heartbeat = "169.254.251.1"
		temp_clc._prop.ext_address = "169.254.240.1/24"
		temp_clc._prop.ext_gateway = "169.254.240.254"
		temp_clc._prop.ext_heartbeat = "169.254.241.1"

		self._prop.channels.append(temp_clc)
		self._update_channels()

	def remove_channel(self, channel=None):
		""" Remove channel from list of channels
		"""
		if channel is not None:
			self._prop.channels.remove(channel)
			self._update_channels()

	id = property(lambda self: self._prop.id, doc="Id")
	flags = property(lambda self: self._prop.flags, doc="Flags")
	name = property(lambda self: self._prop.name, doc="Name")
	channels = property(lambda self: self._prop.channels, doc="Channels")



_Prop_ChannelLabelChannel = plutil.Prop('channel', 'vlan', 'flags',
				 'int_address', 'int_gateway', 'int_heartbeat',
				 'ext_address', 'ext_gateway', 'ext_heartbeat')

class ChannelLabelChannel(object):

	__slots__ = ('_prop', '_pldb', '_channellabel')

	def __init__(self, _pldb=None, **props):
		"""
		Should not be instantiated manually, use L{channel_divert_label_list <Channels.channel_divert_label_list>}
		or L{channel_monitor_label_list <Channels.channel_monitor_label_list>} depending on what kind of label.
		"""
		self._pldb = _pldb
		self._prop = _Prop_ChannelLabelChannel(**props)
		self._channellabel = None

	def __repr__(self):
		return "<%s #%d '%s'>" % (self.__class__.__name__, self._prop.vlan, self._prop.channel)

	def update(self, channel=None, vlan=None, flags=None,
		   int_address=None, int_heartbeat=None, int_gateway=None,
		   ext_address=None, ext_heartbeat=None, ext_gateway=None):

		""" Update properties on this ChannelLabelChannel.
		Omitting an argument or setting it to C{None} will leave that property unchanged.

		"""
		if channel is not None:
			self._prop.channel = channel
		if vlan is not None:
			self._prop.vlan = vlan
		if flags is not None:
			self._prop.flags = flags
		if int_address is not None:
			self._prop.int_address = int_address
		if int_heartbeat is not None:
			self._prop.int_heartbeat = int_heartbeat
		if int_gateway is not None:
			self._prop.int_gateway = int_gateway
		if ext_address is not None:
			self._prop.ext_address = ext_address
		if ext_heartbeat is not None:
			self._prop.ext_heartbeat = ext_heartbeat
		if ext_gateway is not None:
			self._prop.ext_gateway = ext_gateway

		self._channellabel._update_channels()


	channel = property(lambda self: self._prop.channel,
			   doc="Channel")
	vlan = property(lambda self: self._prop.vlan,
			doc="vlan")
	flags = property(lambda self: self._prop.flags,
			doc="flags")
	int_address = property(lambda self: self._prop.int_address,
			doc="internal vlan IP")
	int_gateway = property(lambda self: self._prop.int_gateway,
			doc="internal gateway")
	int_heartbeat = property(lambda self: self._prop.int_heartbeat,
			doc="internal heartbeat")
	ext_address = property(lambda self: self._prop.ext_address,
			doc="external vlan IP")
	ext_gateway = property(lambda self: self._prop.ext_gateway,
			doc="external gateway")
	ext_heartbeat = property(lambda self: self._prop.ext_heartbeat,
			doc="external heartbeat")


class Channels(Resource):
	""" This is the resource for Channel inquiry and manipulation.
	Changes needs to be commited to take effect.

	@cvar CHANNEL_MEDIA_AUTO: Media type is Auto Negotiated
	@cvar CHANNEL_MEDIA_10HD: Media type is 10 Mbps Half Duplex
	@cvar CHANNEL_MEDIA_10FD: Media type is 10 Mbps Full Duplex
	@cvar CHANNEL_MEDIA_100HD: Media type is 100 Mbps Half Duplex
	@cvar CHANNEL_MEDIA_100FD: Media type is 100 Mbps Full Duplex
	@cvar CHANNEL_MEDIA_1000FD: Media type is 1 Gbps Full Duplex
	@cvar CHANNEL_MEDIA_10000FD: Media type is 10 Gbps Full Duplex
	@cvar CHANNEL_MEDIA_40000FD: Media type is 40 Gbps Full Duplex
	@cvar CHANNEL_MEDIA_100000FD: Media type is 100 Gbps Full Duplex

        @cvar ROLE_NONE: Channel is not used
        @cvar ROLE_TRAFFIC: Channel used for passing traffic
        @cvar ROLE_DIVERT: Channel used for divert
        @cvar ROLE_MONITOR: Channel used for sending out monitored traffic
        @cvar ROLE_FLOWSYNC: Channel used for flowsync

        @cvar CAPS_AUTONEG: Channel can do auto negotiation
        @cvar CAPS_10HD: Channel can do 10 Mbps Half Duplex
        @cvar CAPS_10FD: Channel can do 10 Mbps Full Duplex
        @cvar CAPS_100HD: Channel can do 100 Mbps Half Duplex
        @cvar CAPS_100FD: Channel can do 100 Mbps Full Duplex
        @cvar CAPS_1000FD: Channel can do 1 Gbps Full Duplex
        @cvar CAPS_10000FD: Channel can do 10 Gbps Full Duplex
        @cvar CAPS_40000FD: Channel can do 40 Gbps Full Duplex
        @cvar CAPS_100000FD: Channel can do 100 Gbps Full Duplex

        @cvar CAPS_TRAFFIC: Channel can be used for passing traffic
        @cvar CAPS_DIVERT: Channel can be used for divert
        @cvar CAPS_MONITOR: Channel can be used for sending out monitored traffic
        @cvar CAPS_FLOWSYNC: Channel can be used for flowsync
        @cvar CAPS_LINK_TOGGLE: Channel link can be toggled on/off

        @cvar DIRECTION_DEFAULT: Channel is working in the default directions.
        @cvar DIRECTION_REVERSE: Channel is reversed.
        @cvar DIRECTION_INTERNAL: Both ports in the channel is internal
        @cvar DIRECTION_EXTERNAL: Both ports in the channel is external

	@group Channels: channel_list channel_divert_label_list channel_monitor_label_list

	"""
        CHANNEL_MEDIA_AUTO = PLDB.CHANNEL_MEDIA_AUTO
        CHANNEL_MEDIA_10HD = PLDB.CHANNEL_MEDIA_10HD
        CHANNEL_MEDIA_10FD = PLDB.CHANNEL_MEDIA_10FD
        CHANNEL_MEDIA_100HD = PLDB.CHANNEL_MEDIA_100HD
        CHANNEL_MEDIA_100FD = PLDB.CHANNEL_MEDIA_100FD
        CHANNEL_MEDIA_1000FD = PLDB.CHANNEL_MEDIA_1000FD
        CHANNEL_MEDIA_10000FD = PLDB.CHANNEL_MEDIA_10000FD
        CHANNEL_MEDIA_40000FD = PLDB.CHANNEL_MEDIA_40000FD
        CHANNEL_MEDIA_100000FD = PLDB.CHANNEL_MEDIA_100000FD

        ROLE_NONE = PLDB.CHANNEL_ROLE_NONE
        ROLE_TRAFFIC = PLDB.CHANNEL_ROLE_TRAFFIC
        ROLE_DIVERT = PLDB.CHANNEL_ROLE_DIVERT
        ROLE_MONITOR = PLDB.CHANNEL_ROLE_MONITOR
        ROLE_FLOWSYNC = PLDB.CHANNEL_ROLE_FLOWSYNC

        CAPS_AUTONEG = PLDB.CHANNEL_CAPS_AUTONEG
        CAPS_10HD = PLDB.CHANNEL_CAPS_10HD
        CAPS_10FD = PLDB.CHANNEL_CAPS_10FD
        CAPS_100HD = PLDB.CHANNEL_CAPS_100HD
        CAPS_100FD = PLDB.CHANNEL_CAPS_100FD
        CAPS_1000FD = PLDB.CHANNEL_CAPS_1000FD
        CAPS_10000FD = PLDB.CHANNEL_CAPS_10000FD
        CAPS_40000FD = PLDB.CHANNEL_CAPS_40000FD
        CAPS_100000FD = PLDB.CHANNEL_CAPS_100000FD

        CAPS_TRAFFIC = PLDB.CHANNEL_CAPS_TRAFFIC
        CAPS_DIVERT = PLDB.CHANNEL_CAPS_DIVERT
        CAPS_MONITOR = PLDB.CHANNEL_CAPS_MONITOR
        CAPS_FLOWSYNC = PLDB.CHANNEL_CAPS_FLOWSYNC
        CAPS_LINK_TOGGLE = PLDB.CHANNEL_CAPS_LINK_TOGGLE

        DIRECTION_DEFAULT = PLDB.CHANNEL_DIRECTION_DEFAULT
        DIRECTION_REVERSE = PLDB.CHANNEL_DIRECTION_REVERSE
        DIRECTION_INTERNAL = PLDB.CHANNEL_DIRECTION_INTERNAL
        DIRECTION_EXTERNAL = PLDB.CHANNEL_DIRECTION_EXTERNAL

	def _resource(self):
		return PLDB.RESOURCE_CHANNELS

	def channel_list(self):
		"""
		List all Channels (Traffic Interface Pairs) in the PacketLogic system.

			>>> ch.channel_list()
			[<Channel #299 'Slot 4/GE.5 (Reserved)'>,
			 <Channel #216 'Slot 4/GE.4'>,
			 <Channel #212 'Slot 4/GE.3'>,
			 <Channel #208 'Slot 4/GE.2'>,
			 <Channel #204 'Slot 4/GE.1'>,
			 <Channel #200 'Slot 4/10GE'>,
			 <Channel #199 'Slot 3/GE.5 (Monitor/FlowSync)'>,
			 <Channel #116 'Slot 3/GE.4'>,
			 <Channel #112 'Slot 3/GE.3'>,
			 <Channel #108 'Slot 3/GE.2'>,
			 <Channel #104 'Slot 3/GE.1'>,
			 <Channel #100 'Slot 3/10GE'>]


		@rtype: C{list} of L{Channel}
		@returns: List containing a L{Channel} object for each channel available on the system.
		"""

		return plutil._do(self, self.session.channel_list(),
				  plutil._ListCB(self, self.session.channel_list_handle, Channel))

	def channel_divert_label_list(self):
		"""
		List all ChannelLabels (divert label) in the PacketLogic system.

			>>> ch.channel_divert_label_list()
			[<ChannelLabel fancydivertlabel>]


		@rtype: C{list} of L{Channel}
		@returns: List containing a L{Channel} object for each channel available on the system.
		"""

		return plutil._do(self, self.session.channel_divert_label_list(),
				  plutil._ListCB(self, self.session.channel_label_list_handle, (ChannelLabel, ChannelLabelChannel)))

	def channel_monitor_label_list(self):
		"""
		List all ChannelsLabels (monitor label) in the PacketLogic system.

			>>> ch.channel_monitor_label_list()
			[<ChannelLabel fancymonitorlabel>]


		@rtype: C{list} of L{Channel}
		@returns: List containing a L{Channel} object for each channel available on the system.
		"""

		return plutil._do(self, self.session.channel_monitor_label_list(),
				  plutil._ListCB(self, self.session.channel_label_list_handle, (ChannelLabel, ChannelLabelChannel)))

	def channel_label_add(self, name="", role=None):
		"""
		Create ChannelLabel with given name and role.

		@type name: C{char}
		@param name: Name of ChannelLabel.

		@type role: C{int}
		@param role: One of the ROLE_DIVERT or ROLE_MONITOR constants
		"""
		if role is None:
			print "Please specify role"
			return

		plutil._do(self, self.session.channel_label_add(role, name, 0),
				  plutil._AddCB(self.session.channel_label_add_handle))

